@@ -172,6 +172,8 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (EmpathyChat, empathy_chat, GTK_TYPE_BIN);
 
+static gboolean update_misspelled_words (gpointer data);
+
 static void
 chat_get_property (GObject    *object,
 		   guint       param_id,
@@ -1763,33 +1765,196 @@ chat_spelling_menu_activate_cb (GtkMenuItem     *menu_item,
                                gtk_menu_item_get_label (menu_item));
 }
 
+
+static GtkWidget *
+chat_spelling_build_suggestions_menu (const gchar *code,
+				      EmpathyChatSpell *chat_spell)
+{
+	GList     *suggestions, *l;
+	GtkWidget *menu, *menu_item;
+
+	suggestions = empathy_spell_get_suggestions (code, chat_spell->word);
+	if (suggestions == NULL)
+		return NULL;
+
+	menu = gtk_menu_new ();
+	for (l = suggestions; l; l = l->next) {
+		menu_item = gtk_menu_item_new_with_label (l->data);
+		g_signal_connect (G_OBJECT (menu_item), "activate",
+				  G_CALLBACK (chat_spelling_menu_activate_cb),
+				  chat_spell);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	}
+	empathy_spell_free_suggestions (suggestions);
+
+	gtk_widget_show_all (menu);
+
+	return menu;
+}
+
 static GtkWidget *
 chat_spelling_build_menu (EmpathyChatSpell *chat_spell)
 {
-    GtkWidget *menu, *menu_item;
-    GList     *suggestions, *l;
+	GtkWidget *menu, *submenu, *item;
+	GList     *codes, *l;
 
-    menu = gtk_menu_new ();
-    suggestions = empathy_spell_get_suggestions (chat_spell->word);
-    if (suggestions == NULL) {
-        menu_item = gtk_menu_item_new_with_label (_("(No Suggestions)"));
-	gtk_widget_set_sensitive (menu_item, FALSE);
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-    } else {
-        for (l = suggestions; l; l = l->next) {
-            menu_item = gtk_menu_item_new_with_label (l->data);
-            g_signal_connect (G_OBJECT (menu_item),
-                          "activate",
-                          G_CALLBACK (chat_spelling_menu_activate_cb),
-                          chat_spell);
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-        }
-    }
-    empathy_spell_free_suggestions (suggestions);
+	codes = empathy_spell_get_enabled_language_codes ();
+	g_assert (codes != NULL);
 
-    gtk_widget_show_all (menu);
+	if (g_list_length (codes) > 1) {
+		menu = gtk_menu_new ();
 
-    return menu;
+		for (l = codes; l; l = l->next) {
+			const gchar *code, *name;
+
+			code = l->data;
+			name = empathy_spell_get_language_name (code);
+			if (!name)
+				continue;
+
+			item = gtk_image_menu_item_new_with_label (name);
+
+			submenu = chat_spelling_build_suggestions_menu (
+					code, chat_spell);
+			if (submenu == NULL)
+				gtk_widget_set_sensitive (item, FALSE);
+			else
+				gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
+							   submenu);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		}
+	} else {
+		menu = chat_spelling_build_suggestions_menu (codes->data,
+							     chat_spell);
+		if (menu == NULL) {
+			menu = gtk_menu_new ();
+			item = gtk_menu_item_new_with_label (_("(No Suggestions)"));
+			gtk_widget_set_sensitive (item, FALSE);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		}
+	}
+	g_list_free (codes);
+
+	gtk_widget_show_all (menu);
+
+	return menu;
+}
+
+typedef struct {
+	EmpathyChat  *chat;
+	gchar        *word;
+	gchar        *code;
+} EmpathyChatWord;
+
+static EmpathyChatWord *
+chat_word_new (EmpathyChat  *chat,
+		const gchar  *word,
+		const gchar  *code)
+{
+	EmpathyChatWord *chat_word;
+
+	chat_word = g_slice_new0 (EmpathyChatWord);
+
+	chat_word->chat = g_object_ref (chat);
+	chat_word->word = g_strdup (word);
+	chat_word->code = g_strdup (code);
+
+	return chat_word;
+}
+
+static void
+chat_word_free (EmpathyChatWord *chat_word)
+{
+	g_object_unref (chat_word->chat);
+	g_free (chat_word->word);
+	g_free (chat_word->code);
+	g_slice_free (EmpathyChatWord, chat_word);
+}
+
+static void
+chat_add_to_dictionary_activate_cb (GtkMenuItem     *menu_item,
+				    EmpathyChatWord *chat_word)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat_word->chat);
+
+	empathy_spell_add_to_dictionary (chat_word->code,
+					 chat_word->word);
+	priv->update_misspelled_words_id =
+		g_idle_add (update_misspelled_words, chat_word->chat);
+}
+
+static GtkWidget *
+chat_spelling_build_add_to_dictionary_item (EmpathyChatSpell *chat_spell)
+{
+	GtkWidget       *menu, *item, *lang_item, *image;
+	GList           *codes, *l;
+	gchar           *label;
+	const gchar     *code, *name;
+	EmpathyChatWord *chat_word;
+
+	codes = empathy_spell_get_enabled_language_codes ();
+	g_assert (codes != NULL);
+	if (g_list_length (codes) > 1) {
+		/* translators: %s is the selected word */
+		label = g_strdup_printf (_("Add '%s' to Dictionary"),
+					chat_spell->word);
+		item = gtk_image_menu_item_new_with_mnemonic (label);
+		g_free (label);
+		image = gtk_image_new_from_icon_name (GTK_STOCK_ADD,
+						      GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+					       image);
+
+		menu = gtk_menu_new ();
+
+		for (l = codes; l; l = l->next) {
+			code = l->data;
+			name = empathy_spell_get_language_name (code);
+			if (name == NULL)
+				continue;
+
+			lang_item = gtk_image_menu_item_new_with_label (name);
+
+			chat_word= chat_word_new (chat_spell->chat,
+						  chat_spell->word, code);
+			g_object_set_data_full (G_OBJECT (lang_item),
+				"chat-word", chat_word,
+				(GDestroyNotify) chat_word_free);
+
+			g_signal_connect (G_OBJECT (lang_item), "activate",
+				G_CALLBACK (chat_add_to_dictionary_activate_cb),
+				chat_word);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), lang_item);
+		}
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
+	} else {
+		code = codes->data;
+		name = empathy_spell_get_language_name (code);
+		g_assert (name != NULL);
+		/* translators: first %s is the selected word,
+		 * second %s is the language name of the target dictionary */
+		label = g_strdup_printf (_("Add '%s' to %s Dictionary"),
+					chat_spell->word, name);
+		item = gtk_image_menu_item_new_with_mnemonic (label);
+		g_free (label);
+		image = gtk_image_new_from_icon_name (GTK_STOCK_ADD,
+						      GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+		chat_word = chat_word_new (chat_spell->chat, chat_spell->word,
+					   code);
+		g_object_set_data_full (G_OBJECT (item), "chat-word", chat_word,
+					(GDestroyNotify) chat_word_free);
+
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (chat_add_to_dictionary_activate_cb),
+				  chat_word);
+	}
+	g_list_free (codes);
+
+	gtk_widget_show_all (item);
+
+	return item;
 }
 
 static void
@@ -1814,6 +1979,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	gchar                *str = NULL;
 	EmpathyChatSpell     *chat_spell;
 	GtkWidget            *spell_menu;
+	GtkWidget            *spell_item;
 	EmpathySmileyManager *smiley_manager;
 	GtkWidget            *smiley_menu;
 	GtkWidget            *image;
@@ -1871,13 +2037,14 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	if (!EMP_STR_EMPTY (str)) {
 		chat_spell = chat_spell_new (chat, str, start, end);
 		g_object_set_data_full (G_OBJECT (menu),
-					"chat_spell", chat_spell,
+					"chat-spell", chat_spell,
 					(GDestroyNotify) chat_spell_free);
 
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
+		/* Spelling suggestions */
 		item = gtk_image_menu_item_new_with_mnemonic (_("_Spelling Suggestions"));
 		image = gtk_image_new_from_icon_name (GTK_STOCK_SPELL_CHECK,
 						      GTK_ICON_SIZE_MENU);
@@ -1886,6 +2053,17 @@ chat_input_populate_popup_cb (GtkTextView *view,
 		spell_menu = chat_spelling_build_menu (chat_spell);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), spell_menu);
 
+
+		spell_item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (spell_menu), spell_item);
+		gtk_widget_show (spell_item);
+
+		/* Add to dictionary */
+		spell_item = chat_spelling_build_add_to_dictionary_item (chat_spell);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (spell_menu), spell_item);
+		gtk_widget_show (spell_item);
+
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 	}
@@ -3207,6 +3385,31 @@ empathy_chat_copy (EmpathyChat *chat)
 
 		gtk_text_buffer_copy_clipboard (buffer, clipboard);
 	}
+	else {
+		gint start_offset;
+		gint end_offset;
+		EmpathyChatPriv *priv = GET_PRIV (chat);
+
+		if (gtk_label_get_selection_bounds (GTK_LABEL (priv->label_topic),
+							       &start_offset,
+							       &end_offset)) {
+			gchar *start;
+			gchar *end;
+			gchar *selection;
+			const gchar *topic;
+			GtkClipboard *clipboard;
+
+			topic = gtk_label_get_text (GTK_LABEL (priv->label_topic));
+			start = g_utf8_offset_to_pointer (topic, start_offset);
+			end = g_utf8_offset_to_pointer (topic, end_offset);
+			selection = g_strndup (start, end - start);
+
+			clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+			gtk_clipboard_set_text (clipboard, selection, -1);
+
+			g_free (selection);
+		}
+	}
 }
 
 void
