@@ -34,6 +34,7 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
@@ -386,8 +387,8 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 {
 	EmpathyContactListViewPriv *priv;
 	TpAccountManager           *account_manager;
-	TpConnection               *connection = NULL;
-	TpAccount                  *account = NULL;
+        TpConnection               *connection;
+	TpAccount                  *account;
 	DndGetContactData          *data;
 	GtkTreePath                *source_path;
 	const gchar   *sel_data;
@@ -823,6 +824,8 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 		gtk_menu_popup (GTK_MENU (menu),
 				NULL, NULL, NULL, NULL,
 				data->button, data->time);
+		g_object_ref_sink (menu);
+		g_object_unref (menu);
 	}
 
 	g_slice_free (MenuPopupData, data);
@@ -853,7 +856,7 @@ contact_list_view_key_press_event_cb (EmpathyContactListView *view,
 				      GdkEventKey	     *event,
 				      gpointer		      user_data)
 {
-	if (event->keyval == GDK_Menu) {
+	if (event->keyval == GDK_KEY_Menu) {
 		MenuPopupData *data;
 
 		data = g_slice_new (MenuPopupData);
@@ -1279,7 +1282,7 @@ contact_list_view_search_key_navigation_cb (GtkWidget *search,
 	GdkEventKey *eventkey = ((GdkEventKey *) event);
 	gboolean ret = FALSE;
 
-	if (eventkey->keyval == GDK_Up || eventkey->keyval == GDK_Down) {
+	if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down) {
 		GdkEvent *new_event;
 
 		new_event = gdk_event_copy (event);
