@@ -231,9 +231,7 @@ empathy_icon_name_for_individual (FolksIndividual *individual)
 	FolksPresenceType folks_presence;
 	TpConnectionPresenceType presence;
 
-	folks_presence =
-	    folks_presence_details_get_presence_type (
-	        FOLKS_PRESENCE_DETAILS (individual));
+	folks_presence = folks_individual_get_presence_type (individual);
 	presence = empathy_folks_presence_type_to_tp (folks_presence);
 
 	return empathy_icon_name_for_presence (presence);
@@ -605,7 +603,7 @@ avatar_file_load_contents_cb (GObject      *object,
 	}
 
 	g_simple_async_result_set_op_res_gpointer (closure->result,
-			avatar_pixbuf_from_loader (loader), g_object_unref);
+			avatar_pixbuf_from_loader (loader), NULL);
 
 out:
 	g_simple_async_result_complete (closure->result);
@@ -633,8 +631,7 @@ empathy_pixbuf_avatar_from_individual_scaled_async (
 			callback, user_data,
 			empathy_pixbuf_avatar_from_individual_scaled_async);
 
-	avatar_file =
-		folks_avatar_details_get_avatar (FOLKS_AVATAR_DETAILS (individual));
+	avatar_file = folks_avatar_get_avatar (FOLKS_AVATAR (individual));
 	if (avatar_file == NULL)
 		goto out;
 
@@ -656,7 +653,6 @@ out:
 	g_object_unref (result);
 }
 
-/* Return a ref on the GdkPixbuf */
 GdkPixbuf *
 empathy_pixbuf_avatar_from_individual_scaled_finish (
 		FolksIndividual *individual,
@@ -665,7 +661,6 @@ empathy_pixbuf_avatar_from_individual_scaled_finish (
 {
 	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
 	gboolean result_valid;
-	GdkPixbuf *pixbuf;
 
 	g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (simple), NULL);
@@ -678,8 +673,7 @@ empathy_pixbuf_avatar_from_individual_scaled_finish (
 			empathy_pixbuf_avatar_from_individual_scaled_async);
 	g_return_val_if_fail (result_valid, NULL);
 
-	pixbuf = g_simple_async_result_get_op_res_gpointer (simple);
-	return pixbuf != NULL ? g_object_ref (pixbuf) : NULL;
+	return g_simple_async_result_get_op_res_gpointer (simple);
 }
 
 GdkPixbuf *
@@ -1576,7 +1570,7 @@ empathy_window_iconify (GtkWindow *window, GtkStatusIcon *status_icon)
 	XChangeProperty (dpy,
 			 GDK_WINDOW_XID (gdk_window),
 			 gdk_x11_get_xatom_by_name_for_display (
-				gdk_drawable_get_display (gdk_window),
+				gdk_window_get_display (gdk_window),
 			 "_NET_WM_ICON_GEOMETRY"),
 			 XA_CARDINAL, 32, PropModeReplace,
 			 (guchar *)&data, 4);
@@ -1840,66 +1834,11 @@ file_manager_receive_file_response_cb (GtkDialog *dialog,
 	GFile *file;
 
 	if (response == GTK_RESPONSE_OK) {
-		GFile *parent;
-		GFileInfo *info;
-		guint64 free_space, file_size;
-		GError *error = NULL;
-
-		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
-		parent = g_file_get_parent (file);
-		info = g_file_query_filesystem_info (parent,
-				G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
-				NULL, &error);
-
-		g_object_unref (parent);
-
-		if (error != NULL) {
-			g_warning ("Error: %s", error->message);
-
-			g_object_unref (file);
-			return;
-		}
-
-		free_space = g_file_info_get_attribute_uint64 (info,
-				G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
-		file_size = empathy_ft_handler_get_total_bytes (handler);
-
-		g_object_unref (info);
-
-		if (file_size > free_space) {
-			GtkWidget *message = gtk_message_dialog_new (
-				GTK_WINDOW (dialog),
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_CLOSE,
-				_("Insufficient free space to save file"));
-			char *file_size_str, *free_space_str;
-
-			file_size_str = g_format_size_for_display (file_size);
-			free_space_str = g_format_size_for_display (free_space);
-
-			gtk_message_dialog_format_secondary_text (
-				GTK_MESSAGE_DIALOG (message),
-				_("%s of free space are required to save this "
-				  "file, but only %s is available. Please "
-				  "choose another location."),
-				file_size_str, free_space_str);
-
-			gtk_dialog_run (GTK_DIALOG (message));
-
-			g_free (file_size_str);
-			g_free (free_space_str);
-			gtk_widget_destroy (message);
-
-			g_object_unref (file);
-
-			return;
-		}
-
 		factory = empathy_ft_factory_dup_singleton ();
+		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
 
-		empathy_ft_factory_set_destination_for_incoming_handler (
-				factory, handler, file);
+		empathy_ft_factory_set_destination_for_incoming_handler
+			(factory, handler, file);
 
 		g_object_unref (factory);
 		g_object_unref (file);
