@@ -28,7 +28,6 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
-#include <telepathy-yell/telepathy-yell.h>
 
 #include <telepathy-logger/log-manager.h>
 
@@ -40,6 +39,7 @@
 #endif
 
 #include "empathy-contact.h"
+#include "empathy-individual-manager.h"
 #include "empathy-utils.h"
 #include "empathy-enum-types.h"
 #include "empathy-marshal.h"
@@ -485,8 +485,8 @@ empathy_contact_set_presence_message (EmpathyContact *contact,
 
   if (priv->persona != NULL)
     {
-      folks_presence_details_set_presence_message (
-          FOLKS_PRESENCE_DETAILS (priv->persona), message);
+      folks_presence_set_presence_message (FOLKS_PRESENCE (priv->persona),
+          message);
     }
 }
 
@@ -704,12 +704,12 @@ empathy_contact_set_alias (EmpathyContact *contact,
 
   /* Set the alias on the persona if possible */
   persona = empathy_contact_get_persona (contact);
-  if (persona != NULL && FOLKS_IS_ALIAS_DETAILS (persona))
+  if (persona != NULL && FOLKS_IS_ALIASABLE (persona))
     {
       DEBUG ("Setting alias for contact %s to %s",
           empathy_contact_get_id (contact), alias);
 
-      folks_alias_details_set_alias (FOLKS_ALIAS_DETAILS (persona), alias);
+      folks_aliasable_set_alias (FOLKS_ALIASABLE (persona), alias);
     }
 
   if (tp_strdiff (alias, priv->alias))
@@ -727,10 +727,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroupDetails *group_details = FOLKS_GROUP_DETAILS (source);
+  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
   GError *error = NULL;
 
-  folks_group_details_change_group_finish (group_details, result, &error);
+  folks_groupable_change_group_finish (groupable, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -754,9 +754,9 @@ empathy_contact_change_group (EmpathyContact *contact, const gchar *group,
   persona = empathy_contact_get_persona (contact);
   if (persona != NULL)
     {
-      if (FOLKS_IS_GROUP_DETAILS (persona))
-        folks_group_details_change_group (FOLKS_GROUP_DETAILS (persona), group,
-            is_member, groups_change_group_cb, contact);
+      if (FOLKS_IS_GROUPABLE (persona))
+        folks_groupable_change_group (FOLKS_GROUPABLE (persona), group, is_member,
+          groups_change_group_cb, contact);
       return;
     }
 
@@ -860,7 +860,7 @@ empathy_contact_get_persona (EmpathyContact *contact)
             {
               TpfPersona *persona = j->data;
 
-              if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
+              if (TPF_IS_PERSONA (persona))
                 {
                   TpContact *tp_contact = tpf_persona_get_contact (persona);
 
@@ -919,8 +919,7 @@ empathy_contact_set_persona (EmpathyContact *contact,
   /* Set the persona's groups */
   if (priv->groups != NULL)
     {
-      folks_group_details_set_groups (FOLKS_GROUP_DETAILS (persona),
-          priv->groups);
+      folks_groupable_set_groups (FOLKS_GROUPABLE (persona), priv->groups);
       g_hash_table_destroy (priv->groups);
       priv->groups = NULL;
     }
@@ -967,13 +966,7 @@ empathy_contact_get_presence_message (EmpathyContact *contact)
   priv = GET_PRIV (contact);
 
   if (priv->persona != NULL)
-    {
-      return folks_presence_details_get_presence_message (
-          FOLKS_PRESENCE_DETAILS (priv->persona));
-    }
-
-  if (priv->tp_contact != NULL)
-    return tp_contact_get_presence_message (priv->tp_contact);
+    return folks_presence_get_presence_message (FOLKS_PRESENCE (priv->persona));
 
   return NULL;
 }
@@ -1139,17 +1132,12 @@ static gboolean
 contact_has_log (EmpathyContact *contact)
 {
   TplLogManager *manager;
-  TplEntity *entity;
   gboolean have_log;
 
   manager = tpl_log_manager_dup_singleton ();
-  entity = tpl_entity_new (empathy_contact_get_id (contact),
-      TPL_ENTITY_CONTACT, NULL, NULL);
-
   have_log = tpl_log_manager_exists (manager,
-      empathy_contact_get_account (contact), entity, TPL_EVENT_MASK_TEXT);
-
-  g_object_unref (entity);
+      empathy_contact_get_account (contact), empathy_contact_get_id (contact),
+      FALSE);
   g_object_unref (manager);
 
   return have_log;
@@ -1674,29 +1662,6 @@ tp_caps_to_capabilities (TpCapabilities *caps)
                 capabilities |= EMPATHY_CAPABILITIES_VIDEO;
             }
         }
-      else if (!tp_strdiff (chan_type,
-        TPY_IFACE_CHANNEL_TYPE_CALL))
-        {
-          guint j;
-
-          if (tp_asv_get_boolean (fixed_prop,
-              TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL))
-            capabilities |= EMPATHY_CAPABILITIES_AUDIO;
-
-          if (tp_asv_get_boolean (fixed_prop,
-              TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL))
-            capabilities |= EMPATHY_CAPABILITIES_VIDEO;
-
-          for (j = 0; allowed_prop[j] != NULL; j++)
-            {
-              if (!tp_strdiff (allowed_prop[j],
-                    TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO))
-                capabilities |= EMPATHY_CAPABILITIES_AUDIO;
-              else if (!tp_strdiff (allowed_prop[j],
-                    TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO))
-                capabilities |= EMPATHY_CAPABILITIES_VIDEO;
-            }
-        }
     }
 
   return capabilities;
@@ -1776,16 +1741,15 @@ static int
 presence_cmp_func (EmpathyContact *a,
     EmpathyContact *b)
 {
-  FolksPresenceDetails *presence_a, *presence_b;
+  FolksPresence *presence_a, *presence_b;
 
-  presence_a = FOLKS_PRESENCE_DETAILS (empathy_contact_get_persona (a));
-  presence_b = FOLKS_PRESENCE_DETAILS (empathy_contact_get_persona (b));
+  presence_a = FOLKS_PRESENCE (empathy_contact_get_persona (a));
+  presence_b = FOLKS_PRESENCE (empathy_contact_get_persona (b));
 
   /* We negate the result because we're sorting in reverse order (i.e. such that
    * the Personas with the highest presence are at the beginning of the list. */
-  return -folks_presence_details_typecmp (
-      folks_presence_details_get_presence_type (presence_a),
-      folks_presence_details_get_presence_type (presence_b));
+  return -folks_presence_typecmp (folks_presence_get_presence_type (presence_a),
+      folks_presence_get_presence_type (presence_b));
 }
 
 static gint
@@ -1939,7 +1903,7 @@ empathy_contact_dup_best_for_action (FolksIndividual *individual,
       TpContact *tp_contact;
       EmpathyContact *contact;
 
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
+      if (!TPF_IS_PERSONA (l->data))
         continue;
 
       tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
