@@ -39,7 +39,6 @@
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-connectivity.h>
-#include <libempathy/empathy-gsettings.h>
 
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-protocol-chooser.h>
@@ -106,18 +105,12 @@ typedef struct {
   GtkWidget *spinner;
   gboolean loading;
 
-  /* We have to keep a weak reference on the actual EmpathyAccountWidget, not
-   * just its GtkWidget. It is the only reliable source we can query to know if
+  /* We have to keep a reference on the actual EmpathyAccountWidget, not just
+   * his GtkWidget. It is the only reliable source we can query to know if
    * there are any unsaved changes to the currently selected account. We can't
    * look at the account settings because it does not contain everything that
    * can be changed using the EmpathyAccountWidget. For instance, it does not
-   * contain the state of the "Enabled" checkbox.
-   *
-   * Even if we create it ourself, we just get a weak ref and not a strong one
-   * as EmpathyAccountWidget unrefs itself when the GtkWidget is destroyed.
-   * That's kinda ugly; cf bgo #640417.
-   *
-   * */
+   * contain the state of the "Enabled" checkbox. */
   EmpathyAccountWidget *setting_widget_object;
 
   gboolean  connecting_show;
@@ -491,16 +484,9 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   const gchar               *icon_name;
   TpAccount                 *account;
 
-  if (priv->setting_widget_object != NULL)
-    g_object_remove_weak_pointer (G_OBJECT (priv->setting_widget_object),
-        (gpointer *) &priv->setting_widget_object);
-
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
 
-  g_object_add_weak_pointer (G_OBJECT (priv->setting_widget_object),
-      (gpointer *) &priv->setting_widget_object);
-
   if (accounts_dialog_has_valid_accounts (dialog))
     empathy_account_widget_set_other_accounts_exist (
         priv->setting_widget_object, TRUE);
@@ -597,68 +583,18 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
 {
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   EmpathyAccountSettings *settings;
-  gchar *str;
-  const gchar *name, *display_name;
-  TpConnectionManager *cm;
-  TpConnectionManagerProtocol *proto;
-  gboolean is_gtalk, is_facebook;
 
-  cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk,
-      &is_facebook);
-  if (cm == NULL)
+  settings = empathy_protocol_chooser_create_account_settings (
+      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol));
+  if (settings == NULL)
     return;
 
-  if (is_gtalk)
-    name = "gtalk";
-  else if (is_facebook)
-    name ="facebook";
-  else
-    name = proto->name;
-
-  display_name = empathy_protocol_name_to_display_name (name);
-  if (display_name == NULL)
-    display_name = proto->name;
-
-  /* Create account */
-  /* To translator: %s is the name of the protocol, such as "Google Talk" or
-   * "Yahoo!"
-   */
-  str = g_strdup_printf (_("New %s account"), display_name);
-  settings = empathy_account_settings_new (cm->name, proto->name, str);
-
-  g_free (str);
-
-  if (is_gtalk)
-    {
-      gchar *fallback_servers[] = {
-          "talkx.l.google.com",
-          "talkx.l.google.com:443,oldssl",
-          "talkx.l.google.com:80",
-          NULL};
-
-      empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
-          NULL, NULL);
-
-      empathy_account_settings_set_strv (settings, "fallback-servers",
-          fallback_servers);
-    }
-  else if (is_facebook)
-    {
-      empathy_account_settings_set_icon_name_async (settings, "im-facebook",
-          NULL, NULL);
-
-      empathy_account_settings_set_string (settings, "server",
-          "chat.facebook.com");
-    }
-
   accounts_dialog_add (dialog, settings);
   accounts_dialog_model_set_selected (dialog, settings);
 
   gtk_widget_show_all (priv->hbox_protocol);
 
   g_object_unref (settings);
-  g_object_unref (cm);
 }
 
 static void
@@ -1038,22 +974,8 @@ accounts_dialog_row_changed_foreach (GtkTreeModel *model,
     GtkTreeIter *iter,
     gpointer user_data)
 {
-  TpAccount *account;
+  gtk_tree_model_row_changed (model, path, iter);
 
-  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
-
-  if (account == NULL)
-    return FALSE;
-
-  if (tp_account_get_connection_status (account, NULL) ==
-      TP_CONNECTION_STATUS_CONNECTING)
-    {
-      /* Only update the row where we have a connecting account as that's the
-       * ones having a blinking icon. */
-      gtk_tree_model_row_changed (model, path, iter);
-    }
-
-  g_object_unref (account);
   return FALSE;
 }
 
@@ -2283,7 +2205,6 @@ do_constructed (GObject *object)
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (object);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   GtkTreeModel *model;
-  GSettings *gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
 
   accounts_dialog_build_ui (dialog);
   accounts_dialog_model_setup (dialog);
@@ -2300,24 +2221,7 @@ do_constructed (GObject *object)
   tp_account_manager_prepare_async (priv->account_manager, NULL,
       accounts_dialog_manager_ready_cb, dialog);
 
-  if (empathy_import_accounts_to_import ())
-    {
-      gtk_widget_show (priv->button_import);
-
-      if (!g_settings_get_boolean (gsettings, EMPATHY_PREFS_IMPORT_ASKED))
-        {
-          GtkWidget *import_dialog;
-
-          g_settings_set_boolean (gsettings, EMPATHY_PREFS_IMPORT_ASKED, TRUE);
-          import_dialog = empathy_import_dialog_new (GTK_WINDOW (dialog),
-              FALSE);
-          gtk_widget_show (import_dialog);
-        }
-    }
-
   priv->connectivity = empathy_connectivity_dup_singleton ();
-
-  g_object_unref (gsettings);
 }
 
 static void
