@@ -21,6 +21,7 @@
 
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-utils.h>
 
 #include "empathy-chat-window.h"
 
@@ -30,7 +31,8 @@
 #include "empathy-chat-manager.h"
 
 enum {
-  CHATS_CHANGED,
+  CLOSED_CHATS_CHANGED,
+  HANDLED_CHATS_CHANGED,
   LAST_SIGNAL
 };
 
@@ -43,8 +45,11 @@ typedef struct _EmpathyChatManagerPriv EmpathyChatManagerPriv;
 
 struct _EmpathyChatManagerPriv
 {
+  EmpathyChatroomManager *chatroom_mgr;
   /* Queue of (ChatData *) representing the closed chats */
-  GQueue *queue;
+  GQueue *closed_queue;
+
+  guint num_handled_channels;
 
   TpBaseClient *handler;
 };
@@ -95,32 +100,31 @@ chat_data_free (ChatData *data)
 }
 
 static void
-tell_chatroom_manager_if_needed (TpAccount *account,
+tell_chatroom_manager_if_needed (EmpathyChatManager *self,
+    TpAccount *account,
     EmpathyTpChat *chat)
 {
+  EmpathyChatManagerPriv *priv = GET_PRIV (self);
   TpHandleType type;
 
   tp_channel_get_handle (empathy_tp_chat_get_channel (chat), &type);
 
   if (type == TP_HANDLE_TYPE_ROOM)
     {
-      EmpathyChatroomManager *chatroom_mgr;
-
-      chatroom_mgr = empathy_chatroom_manager_dup_singleton (NULL);
-      empathy_chatroom_manager_chat_handled (chatroom_mgr, chat, account);
-      g_object_unref (chatroom_mgr);
+      empathy_chatroom_manager_chat_handled (priv->chatroom_mgr, chat, account);
     }
 }
 
 static void
-process_tp_chat (EmpathyTpChat *tp_chat,
+process_tp_chat (EmpathyChatManager *self,
+    EmpathyTpChat *tp_chat,
     TpAccount *account,
     gint64 user_action_time)
 {
   EmpathyChat *chat = NULL;
   const gchar *id;
 
-  tell_chatroom_manager_if_needed (account, tp_chat);
+  tell_chatroom_manager_if_needed (self, account, tp_chat);
 
   id = empathy_tp_chat_get_id (tp_chat);
   if (!tp_str_empty (id))
@@ -153,6 +157,7 @@ process_tp_chat (EmpathyTpChat *tp_chat,
 
 typedef struct
 {
+  EmpathyChatManager *self;
   EmpathyTpChat *tp_chat;
   TpAccount *account;
   gint64 user_action_time;
@@ -160,12 +165,14 @@ typedef struct
 } chat_ready_ctx;
 
 static chat_ready_ctx *
-chat_ready_ctx_new (EmpathyTpChat *tp_chat,
+chat_ready_ctx_new (EmpathyChatManager *self,
+    EmpathyTpChat *tp_chat,
     TpAccount *account,
     gint64 user_action_time)
 {
   chat_ready_ctx *ctx = g_slice_new0 (chat_ready_ctx);
 
+  ctx->self = g_object_ref (self);
   ctx->tp_chat = g_object_ref (tp_chat);
   ctx->account = g_object_ref (account);
   ctx->user_action_time = user_action_time;
@@ -175,6 +182,7 @@ chat_ready_ctx_new (EmpathyTpChat *tp_chat,
 static void
 chat_ready_ctx_free (chat_ready_ctx *ctx)
 {
+  g_object_unref (ctx->self);
   g_object_unref (ctx->tp_chat);
   g_object_unref (ctx->account);
 
@@ -195,11 +203,29 @@ tp_chat_ready_cb (GObject *object,
   if (!empathy_tp_chat_is_ready (tp_chat))
     return;
 
-  process_tp_chat (tp_chat, ctx->account, ctx->user_action_time);
+  process_tp_chat (ctx->self, tp_chat, ctx->account, ctx->user_action_time);
 
   chat_ready_ctx_free (ctx);
 }
 
+static void
+channel_invalidated (TpChannel *channel,
+    guint domain,
+    gint code,
+    gchar *message,
+    EmpathyChatManager *self)
+{
+  EmpathyChatManagerPriv *priv = GET_PRIV (self);
+
+  priv->num_handled_channels--;
+
+  DEBUG ("Channel closed; we are now handling %u text channels",
+      priv->num_handled_channels);
+
+  g_signal_emit (self, signals[HANDLED_CHATS_CHANGED], 0,
+      priv->num_handled_channels);
+}
+
 static void
 handle_channels (TpSimpleHandler *handler,
     TpAccount *account,
@@ -210,30 +236,52 @@ handle_channels (TpSimpleHandler *handler,
     TpHandleChannelsContext *context,
     gpointer user_data)
 {
+  EmpathyChatManager *self = (EmpathyChatManager *) user_data;
+  EmpathyChatManagerPriv *priv = GET_PRIV (self);
   GList *l;
+  gboolean handling = FALSE;
 
   for (l = channels; l != NULL; l = g_list_next (l))
     {
       TpChannel *channel = l->data;
       EmpathyTpChat *tp_chat;
 
-      tp_chat = empathy_tp_chat_new (channel);
+      if (tp_proxy_get_invalidated (channel) != NULL)
+        continue;
+
+      handling = TRUE;
+
+      tp_chat = empathy_tp_chat_new (account, channel);
 
       if (empathy_tp_chat_is_ready (tp_chat))
         {
-          process_tp_chat (tp_chat, account, user_action_time);
+          process_tp_chat (self, tp_chat, account, user_action_time);
         }
       else
         {
-          chat_ready_ctx *ctx = chat_ready_ctx_new (tp_chat, account,
+          chat_ready_ctx *ctx = chat_ready_ctx_new (self, tp_chat, account,
               user_action_time);
 
           ctx->sig_id = g_signal_connect (tp_chat, "notify::ready",
               G_CALLBACK (tp_chat_ready_cb), ctx);
         }
+
+      priv->num_handled_channels++;
+
+      g_signal_connect (channel, "invalidated",
+          G_CALLBACK (channel_invalidated), self);
     }
 
   tp_handle_channels_context_accept (context);
+
+  if (handling)
+    {
+      DEBUG ("Channels handled; we are now handling %u text channels",
+          priv->num_handled_channels);
+
+      g_signal_emit (self, signals[HANDLED_CHATS_CHANGED], 0,
+          priv->num_handled_channels);
+    }
 }
 
 static void
@@ -243,7 +291,7 @@ empathy_chat_manager_init (EmpathyChatManager *self)
   TpDBusDaemon *dbus;
   GError *error = NULL;
 
-  priv->queue = g_queue_new ();
+  priv->closed_queue = g_queue_new ();
 
   dbus = tp_dbus_daemon_dup (&error);
   if (dbus == NULL)
@@ -253,9 +301,15 @@ empathy_chat_manager_init (EmpathyChatManager *self)
       return;
     }
 
+  priv->chatroom_mgr = empathy_chatroom_manager_dup_singleton (NULL);
+
   /* Text channels handler */
-  priv->handler = tp_simple_handler_new (dbus, FALSE, FALSE, "Empathy", FALSE,
-      handle_channels, self, NULL);
+  priv->handler = tp_simple_handler_new (dbus, FALSE, FALSE, "Empathy.Chat",
+      FALSE, handle_channels, self, NULL);
+
+  /* EmpathyTpChat relies on this feature being prepared */
+  tp_base_client_add_connection_features_varargs (priv->handler,
+    TP_CONNECTION_FEATURE_CAPABILITIES, 0);
 
   g_object_unref (dbus);
 
@@ -287,14 +341,15 @@ empathy_chat_manager_finalize (GObject *object)
   EmpathyChatManager *self = EMPATHY_CHAT_MANAGER (object);
   EmpathyChatManagerPriv *priv = GET_PRIV (self);
 
-  if (priv->queue != NULL)
+  if (priv->closed_queue != NULL)
     {
-      g_queue_foreach (priv->queue, (GFunc) chat_data_free, NULL);
-      g_queue_free (priv->queue);
-      priv->queue = NULL;
+      g_queue_foreach (priv->closed_queue, (GFunc) chat_data_free, NULL);
+      g_queue_free (priv->closed_queue);
+      priv->closed_queue = NULL;
     }
 
   tp_clear_object (&priv->handler);
+  tp_clear_object (&priv->chatroom_mgr);
 
   G_OBJECT_CLASS (empathy_chat_manager_parent_class)->finalize (object);
 }
@@ -331,8 +386,18 @@ empathy_chat_manager_class_init (
   object_class->finalize = empathy_chat_manager_finalize;
   object_class->constructor = empathy_chat_manager_constructor;
 
-  signals[CHATS_CHANGED] =
-    g_signal_new ("chats-changed",
+  signals[CLOSED_CHATS_CHANGED] =
+    g_signal_new ("closed-chats-changed",
+        G_TYPE_FROM_CLASS (object_class),
+        G_SIGNAL_RUN_LAST,
+        0,
+        NULL, NULL,
+        g_cclosure_marshal_VOID__UINT,
+        G_TYPE_NONE,
+        1, G_TYPE_UINT, NULL);
+
+  signals[HANDLED_CHATS_CHANGED] =
+    g_signal_new ("handled-chats-changed",
         G_TYPE_FROM_CLASS (object_class),
         G_SIGNAL_RUN_LAST,
         0,
@@ -360,50 +425,13 @@ empathy_chat_manager_closed_chat (EmpathyChatManager *self,
 
   data = chat_data_new (chat);
 
-  DEBUG ("Adding %s to queue: %s", data->room ? "room" : "contact", data->id);
+  DEBUG ("Adding %s to closed queue: %s",
+      data->room ? "room" : "contact", data->id);
 
-  g_queue_push_tail (priv->queue, data);
+  g_queue_push_tail (priv->closed_queue, data);
 
-  g_signal_emit (self, signals[CHATS_CHANGED], 0,
-      g_queue_get_length (priv->queue));
-}
-
-static void
-connection_ready_cb (TpConnection *connection,
-    const GError *error,
-    gpointer user_data)
-{
-  ChatData *data = user_data;
-  EmpathyChatManager *self = chat_manager_singleton;
-  EmpathyChatManagerPriv *priv;
-
-  /* Extremely unlikely to happen, but I don't really want to keep refs to the
-   * chat manager in the ChatData structs as it'll then prevent the manager
-   * from being finalized. */
-  if (G_UNLIKELY (self == NULL))
-    goto out;
-
-  priv = GET_PRIV (self);
-
-  if (error == NULL)
-    {
-      if (data->room)
-        empathy_dispatcher_join_muc (connection, data->id,
-          EMPATHY_DISPATCHER_NON_USER_ACTION);
-      else
-        empathy_dispatcher_chat_with_contact_id (connection, data->id,
-            EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
-
-      g_signal_emit (self, signals[CHATS_CHANGED], 0,
-          g_queue_get_length (priv->queue));
-    }
-  else
-    {
-      DEBUG ("Error readying connection, no chat: %s", error->message);
-    }
-
-out:
-  chat_data_free (data);
+  g_signal_emit (self, signals[CLOSED_CHATS_CHANGED], 0,
+      g_queue_get_length (priv->closed_queue));
 }
 
 void
@@ -411,33 +439,40 @@ empathy_chat_manager_undo_closed_chat (EmpathyChatManager *self)
 {
   EmpathyChatManagerPriv *priv = GET_PRIV (self);
   ChatData *data;
-  TpConnection *connection;
 
-  data = g_queue_pop_tail (priv->queue);
+  data = g_queue_pop_tail (priv->closed_queue);
 
   if (data == NULL)
     return;
 
-  DEBUG ("Removing %s from queue and starting a chat with: %s",
+  DEBUG ("Removing %s from closed queue and starting a chat with: %s",
       data->room ? "room" : "contact", data->id);
 
-  connection = tp_account_get_connection (data->account);
-
-  if (connection != NULL)
-    {
-      tp_connection_call_when_ready (connection, connection_ready_cb, data);
-    }
+  if (data->room)
+    empathy_dispatcher_join_muc (data->account, data->id,
+        TP_USER_ACTION_TIME_NOT_USER_ACTION);
   else
-    {
-      DEBUG ("No connection, no chat.");
-      chat_data_free (data);
-    }
+    empathy_dispatcher_chat_with_contact_id (data->account, data->id,
+        TP_USER_ACTION_TIME_NOT_USER_ACTION);
+
+  g_signal_emit (self, signals[CLOSED_CHATS_CHANGED], 0,
+      g_queue_get_length (priv->closed_queue));
+
+  chat_data_free (data);
 }
 
 guint
-empathy_chat_manager_get_num_chats (EmpathyChatManager *self)
+empathy_chat_manager_get_num_closed_chats (EmpathyChatManager *self)
 {
   EmpathyChatManagerPriv *priv = GET_PRIV (self);
 
-  return g_queue_get_length (priv->queue);
+  return g_queue_get_length (priv->closed_queue);
+}
+
+guint
+empathy_chat_manager_get_num_handled_chats (EmpathyChatManager *self)
+{
+  EmpathyChatManagerPriv *priv = GET_PRIV (self);
+
+  return priv->num_handled_channels;
 }
