@@ -29,17 +29,14 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/simple-approver.h>
 
-#include <telepathy-yell/telepathy-yell.h>
-
 #include <libempathy/empathy-idle.h>
-
-#include <libempathy/empathy-channel-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-streamed-media.h>
+#include <libempathy/empathy-tp-call.h>
 #include <libempathy/empathy-tp-file.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-gsettings.h>
 
 #include <extensions/extensions.h>
@@ -79,7 +76,6 @@ typedef struct {
 
 typedef struct {
   TpBaseClient *approver;
-  TpBaseClient *auth_approver;
   EmpathyContactManager *contact_manager;
   GSList *events;
   /* Ongoing approvals */
@@ -166,8 +162,10 @@ event_free (EventPriv *event)
   if (event->autoremove_timeout_id != 0)
     g_source_remove (event->autoremove_timeout_id);
 
-  tp_clear_object (&(event->public.contact));
-  tp_clear_object (&(event->public.account));
+  if (event->public.contact)
+    {
+      g_object_unref (event->public.contact);
+    }
 
   g_slice_free (EventPriv, event);
 }
@@ -210,7 +208,6 @@ display_notify_area (void)
 
 static void
 event_manager_add (EmpathyEventManager *manager,
-    TpAccount *account,
     EmpathyContact *contact,
     EmpathyEventType type,
     const gchar *icon_name,
@@ -224,7 +221,6 @@ event_manager_add (EmpathyEventManager *manager,
   EventPriv               *event;
 
   event = g_slice_new0 (EventPriv);
-  event->public.account = account != NULL ? g_object_ref (account) : NULL;
   event->public.contact = contact ? g_object_ref (contact) : NULL;
   event->public.type = type;
   event->public.icon_name = g_strdup (icon_name);
@@ -403,16 +399,9 @@ reject_channel_claim_cb (GObject *source,
       goto out;
     }
 
-  if (EMPATHY_IS_TP_STREAMED_MEDIA (user_data))
+  if (EMPATHY_IS_TP_CALL (user_data))
     {
-      empathy_tp_streamed_media_close (user_data);
-    }
-  else if (TPY_IS_CALL_CHANNEL (user_data))
-    {
-      tpy_call_channel_hangup_async (user_data,
-          TPY_CALL_STATE_CHANGE_REASON_USER_REQUESTED,
-          "", "", NULL, NULL);
-      tp_channel_close_async (user_data, NULL, NULL);
+      empathy_tp_call_close (user_data);
     }
   else if (EMPATHY_IS_TP_CHAT (user_data))
     {
@@ -427,47 +416,12 @@ out:
   g_object_unref (user_data);
 }
 
-static void
-reject_auth_channel_claim_cb (GObject *source,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  TpChannelDispatchOperation *cdo = TP_CHANNEL_DISPATCH_OPERATION (source);
-  GError *error = NULL;
-
-  if (!tp_channel_dispatch_operation_claim_finish (cdo, result, &error))
-    {
-      DEBUG ("Failed to claim channel: %s", error->message);
-
-      g_error_free (error);
-      return;
-    }
-
-  tp_cli_channel_call_close (TP_CHANNEL (user_data), -1,
-      NULL, NULL, NULL, NULL);
-}
-
 static void
 reject_approval (EventManagerApproval *approval)
 {
   /* We have to claim the channel before closing it */
-
-  /* Unfortunately, we need to special case the auth channels for the
-   * time being as they don't have a wrapper object handler in
-   * approval->handler_instance as they're not actually handled by
-   * this process, so we can just use a noddy callback to call Close()
-   * directly. */
-  if (approval->handler_instance != NULL)
-    {
-      tp_channel_dispatch_operation_claim_async (approval->operation,
-          reject_channel_claim_cb, g_object_ref (approval->handler_instance));
-    }
-  else if (tp_channel_get_channel_type_id (approval->main_channel)
-      == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
-    {
-      tp_channel_dispatch_operation_claim_async (approval->operation,
-          reject_auth_channel_claim_cb, approval->main_channel);
-    }
+  tp_channel_dispatch_operation_claim_async (approval->operation,
+      reject_channel_claim_cb, g_object_ref (approval->handler_instance));
 }
 
 static void
@@ -495,6 +449,7 @@ event_channel_process_voip_func (EventPriv *event)
   GtkWidget *dialog;
   GtkWidget *button;
   GtkWidget *image;
+  EmpathyTpCall *call;
   gboolean video;
   gchar *title;
 
@@ -504,23 +459,9 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  if (event->public.type == EMPATHY_EVENT_TYPE_VOIP)
-    {
-      EmpathyTpStreamedMedia *call;
-      call = EMPATHY_TP_STREAMED_MEDIA (event->approval->handler_instance);
-      video = empathy_tp_streamed_media_has_initial_video (call);
-    }
-  else if (event->public.type == EMPATHY_EVENT_TYPE_CALL)
-    {
-      TpyCallChannel *call;
-      call = TPY_CALL_CHANNEL (event->approval->handler_instance);
-      g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
-    }
-  else
-    {
-      g_warning ("Unknown event type: %d", event->public.type);
-      return;
-    }
+  call = EMPATHY_TP_CALL (event->approval->handler_instance);
+
+  video = empathy_tp_call_has_initial_video (call);
 
   dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -603,9 +544,9 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
     event_update (approval->manager, event, EMPATHY_IMAGE_NEW_MESSAGE, header,
         msg);
   else
-    event_manager_add (approval->manager, NULL, sender,
-        EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_NEW_MESSAGE, header, msg,
-        approval, event_text_channel_process_func, NULL);
+    event_manager_add (approval->manager, sender, EMPATHY_EVENT_TYPE_CHAT,
+        EMPATHY_IMAGE_NEW_MESSAGE, header, msg, approval,
+        event_text_channel_process_func, NULL);
 
   empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
 
@@ -624,8 +565,7 @@ event_manager_approval_done (EventManagerApproval *approval)
 
       channel_type = tp_channel_get_channel_type_id (approval->main_channel);
 
-      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
-          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
+      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
         {
           priv->ringing--;
           if (priv->ringing == 0)
@@ -661,98 +601,25 @@ cdo_invalidated_cb (TpProxy *cdo,
   event_manager_approval_done (approval);
 }
 
-static void
-event_manager_call_state_changed_cb (TpyCallChannel *call,
-  TpyCallState state,
-  TpyCallFlags flags,
-   const GValueArray *call_state_reason,
-  GHashTable *call_state_details,
-  EventManagerApproval *approval)
-{
-  if (state == TPY_CALL_STATE_ENDED)
-    {
-      DEBUG ("Call ended, seems we missed it :/");
-      reject_approval (approval);
-    }
-}
-
-static void
-event_manager_call_channel_got_contact_cb (TpConnection *connection,
-                                 EmpathyContact *contact,
-                                 const GError *error,
-                                 gpointer user_data,
-                                 GObject *object)
-{
-  EventManagerApproval *approval = (EventManagerApproval *) user_data;
-  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-  GtkWidget *window;
-  TpyCallChannel *call;
-  gchar *header;
-  gboolean video;
-
-  call = TPY_CALL_CHANNEL (approval->handler_instance);
-
-  if (error != NULL)
-    {
-      DEBUG ("Can't get the contact for the call.. Rejecting?");
-      reject_approval (approval);
-      return;
-    }
-
-  if (tpy_call_channel_get_state (call, NULL, NULL) == TPY_CALL_STATE_ENDED)
-    {
-      DEBUG ("Call already ended, seems we missed it :/");
-      reject_approval (approval);
-      return;
-    }
-
-  approval->handler = g_signal_connect (call, "state-changed",
-    G_CALLBACK (event_manager_call_state_changed_cb), approval);
-
-  window = empathy_main_window_dup ();
-  approval->contact = g_object_ref (contact);
-
-  g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
-
-  header = g_strdup_printf (
-    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
-    empathy_contact_get_alias (approval->contact));
-
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_CALL,
-      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
-      header, NULL, approval,
-      event_channel_process_voip_func, NULL);
-
-  g_free (header);
-
-  priv->ringing++;
-  if (priv->ringing == 1)
-    empathy_sound_start_playing (window,
-        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
-
-  g_object_unref (window);
-}
-
 static void
 event_manager_media_channel_got_contact (EventManagerApproval *approval)
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
   GtkWidget *window = empathy_main_window_dup ();
   gchar *header;
-  EmpathyTpStreamedMedia *call;
+  EmpathyTpCall *call;
   gboolean video;
 
-  call = EMPATHY_TP_STREAMED_MEDIA (approval->handler_instance);
+  call = EMPATHY_TP_CALL (approval->handler_instance);
 
-  video = empathy_tp_streamed_media_has_initial_video (call);
+  video = empathy_tp_call_has_initial_video (call);
 
   header = g_strdup_printf (
     video ? _("Incoming video call from %s") :_("Incoming call from %s"),
     empathy_contact_get_alias (approval->contact));
 
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_VOIP,
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_VOIP,
       video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
       header, NULL, approval,
       event_channel_process_voip_func, NULL);
@@ -768,7 +635,7 @@ event_manager_media_channel_got_contact (EventManagerApproval *approval)
 }
 
 static void
-event_manager_media_channel_contact_changed_cb (EmpathyTpStreamedMedia *call,
+event_manager_media_channel_contact_changed_cb (EmpathyTpCall *call,
   GParamSpec *param, EventManagerApproval *approval)
 {
   EmpathyContact *contact;
@@ -883,10 +750,9 @@ display_invite_room_dialog (EventManagerApproval *approval)
           tp_channel_get_identifier (approval->main_channel));
     }
 
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_INVITATION,
-      EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg, approval,
-      event_room_channel_process_func, NULL);
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_INVITATION, EMPATHY_IMAGE_GROUP_MESSAGE, msg,
+      invite_msg, approval, event_room_channel_process_func, NULL);
 
   empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
 
@@ -931,9 +797,8 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
   header = g_strdup_printf (_("Incoming file transfer from %s"),
                             empathy_contact_get_alias (approval->contact));
 
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_TRANSFER,
-      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_TRANSFER, EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
       approval, event_channel_process_func, NULL);
 
   /* FIXME better sound for incoming file transfers ?*/
@@ -943,14 +808,8 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
   g_object_unref (window);
 }
 
-static void
-event_manager_auth_process_func (EventPriv *event)
-{
-  empathy_event_approve ((EmpathyEvent *) event);
-}
-
-/* If there is a file-transfer, media, or auth channel consider it as
- * the main one. */
+/* If there is a file-transfer or media channel consider it as the
+ * main one. */
 static TpChannel *
 find_main_channel (GList *channels)
 {
@@ -968,9 +827,7 @@ find_main_channel (GList *channels)
       channel_type = tp_channel_get_channel_type_id (channel);
 
       if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
-          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL ||
-          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER ||
-          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
+          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
         return channel;
 
       else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TEXT)
@@ -1062,8 +919,7 @@ approve_channels (TpSimpleApprover *approver,
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
     {
       EmpathyContact *contact;
-      EmpathyTpStreamedMedia *call = empathy_tp_streamed_media_new (account,
-        channel);
+      EmpathyTpCall *call = empathy_tp_call_new (account, channel);
 
       approval->handler_instance = G_OBJECT (call);
 
@@ -1082,19 +938,6 @@ approve_channels (TpSimpleApprover *approver,
         }
 
     }
-  else if (channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
-    {
-      TpyCallChannel *call = TPY_CALL_CHANNEL (channel);
-      const gchar *id;
-
-      approval->handler_instance = g_object_ref (call);
-
-      id = tp_channel_get_identifier (channel);
-
-      empathy_tp_contact_factory_get_from_id (connection, id,
-        event_manager_call_channel_got_contact_cb,
-        approval, NULL, G_OBJECT (self));
-    }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
       TpHandle handle;
@@ -1104,16 +947,10 @@ approve_channels (TpSimpleApprover *approver,
 
       handle = tp_channel_get_handle (channel, NULL);
 
+      connection = tp_channel_borrow_connection (channel);
       empathy_tp_contact_factory_get_from_handle (connection, handle,
         event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (self));
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
-    {
-      event_manager_add (approval->manager, account, NULL, EMPATHY_EVENT_TYPE_AUTH,
-          GTK_STOCK_DIALOG_AUTHENTICATION, tp_account_get_display_name (account),
-          _("Password required"), approval,
-          event_manager_auth_process_func, NULL);
-    }
   else
     {
       GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
@@ -1166,7 +1003,7 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
     }
 
   header = g_strdup_printf (
-      _("%s would like permission to see when you are online"),
+      _("%s would like permission to see when you are available"),
       empathy_contact_get_alias (contact));
 
   if (!EMP_STR_EMPTY (message))
@@ -1174,7 +1011,7 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
   else
     event_msg = NULL;
 
-  event_manager_add (manager, NULL, contact, EMPATHY_EVENT_TYPE_SUBSCRIPTION,
+  event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_SUBSCRIPTION,
       GTK_STOCK_DIALOG_QUESTION, header, event_msg, NULL,
       event_pending_subscribe_func, NULL);
 
@@ -1213,12 +1050,11 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (gsettings,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT))
             {
-              header = g_strdup_printf ("<b>%s</b>",
+              header = g_strdup_printf (_("%s is now offline."),
                   empathy_contact_get_alias (contact));
 
-              event_manager_add (manager, NULL, contact,
-                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
-                  header, _("Disconnected"), NULL, NULL, NULL);
+              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
             }
         }
     }
@@ -1234,12 +1070,11 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (gsettings,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN))
             {
-              header = g_strdup_printf ("<b>%s</b>",
+              header = g_strdup_printf (_("%s is now online."),
                   empathy_contact_get_alias (contact));
 
-              event_manager_add (manager, NULL, contact,
-                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
-                  header, _("Connected"), NULL, NULL, NULL);
+              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
             }
         }
     }
@@ -1302,7 +1137,6 @@ event_manager_finalize (GObject *object)
   g_slist_free (priv->approvals);
   g_object_unref (priv->contact_manager);
   g_object_unref (priv->approver);
-  g_object_unref (priv->auth_approver);
 }
 
 static void
@@ -1341,6 +1175,7 @@ empathy_event_manager_class_init (EmpathyEventManagerClass *klass)
       g_cclosure_marshal_VOID__POINTER,
       G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+
   g_type_class_add_private (object_class, sizeof (EmpathyEventManagerPriv));
 }
 
@@ -1350,7 +1185,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
   TpDBusDaemon *dbus;
-  EmpathyChannelFactory *factory;
   GError *error = NULL;
 
   manager->priv = priv;
@@ -1373,10 +1207,9 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   priv->approver = tp_simple_approver_new (dbus, "Empathy.EventManager", FALSE,
       approve_channels, manager, NULL);
 
-  factory = empathy_channel_factory_new ();
-  tp_base_client_set_channel_factory (priv->approver,
-      TP_CLIENT_CHANNEL_FACTORY (factory));
-  g_object_unref (factory);
+  /* EmpathyTpChat relies on this feature being prepared */
+  tp_base_client_add_connection_features_varargs (priv->approver,
+    TP_CONNECTION_FEATURE_CAPABILITIES, 0);
 
   /* Private text channels */
   tp_base_client_take_approver_filter (priv->approver,
@@ -1407,35 +1240,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
           TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
         TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
         NULL));
-  tp_base_client_take_approver_filter (priv->approver,
-      tp_asv_new (
-        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-          TPY_IFACE_CHANNEL_TYPE_CALL,
-        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
-        NULL));
-
-  /* I don't feel good about doing this, and I'm sorry, but the
-   * capabilities connection feature is added earlier because it's
-   * needed for EmpathyTpChat. If the capabilities feature is required
-   * then preparing an auth channel (which of course appears in the
-   * CONNECTING state) will never be prepared. So the options are
-   * either to create another approver like I've done, or to port
-   * EmpathyTpChat and its users to not depend on the connection being
-   * prepared with capabilities. I chose the former, obviously. :-) */
-
-  priv->auth_approver = tp_simple_approver_new (dbus,
-      "Empathy.AuthEventManager", FALSE, approve_channels, manager,
-      NULL);
-
-  /* SASL auth channels */
-  tp_base_client_take_approver_filter (priv->auth_approver,
-      tp_asv_new (
-        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-          TP_IFACE_CHANNEL_TYPE_SERVER_AUTHENTICATION,
-        TP_PROP_CHANNEL_TYPE_SERVER_AUTHENTICATION_AUTHENTICATION_METHOD,
-          G_TYPE_STRING,
-          TP_IFACE_CHANNEL_INTERFACE_SASL_AUTHENTICATION,
-        NULL));
 
   if (!tp_base_client_register (priv->approver, &error))
     {
@@ -1443,12 +1247,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
       g_error_free (error);
     }
 
-  if (!tp_base_client_register (priv->auth_approver, &error))
-    {
-      DEBUG ("Failed to register auth Approver: %s", error->message);
-      g_error_free (error);
-    }
-
   g_object_unref (dbus);
 }
 
