@@ -881,6 +881,12 @@ ft_manager_response_cb (GtkWidget *widget,
       case RESPONSE_STOP:
         ft_manager_stop (manager);
         break;
+      case GTK_RESPONSE_NONE:
+      case GTK_RESPONSE_DELETE_EVENT:
+        /* Do nothing */
+        break;
+      default:
+        g_assert_not_reached ();
     }
 }
 
@@ -921,8 +927,8 @@ ft_manager_key_press_event_cb (GtkWidget *widget,
                                GdkEventKey *event,
                                gpointer user_data)
 {
-  if ((event->state & GDK_CONTROL_MASK && event->keyval == GDK_w)
-      || event->keyval == GDK_Escape)
+  if ((event->state & GDK_CONTROL_MASK && event->keyval == GDK_KEY_w)
+      || event->keyval == GDK_KEY_Escape)
     {
       gtk_widget_destroy (widget);
       return TRUE;
