@@ -30,7 +30,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <unique/unique.h>
 
 #ifdef HAVE_LIBCHAMPLAIN
 #include <clutter-gtk/clutter-gtk.h>
@@ -64,7 +63,6 @@
 #include "empathy-main-window.h"
 #include "empathy-accounts-common.h"
 #include "empathy-accounts-dialog.h"
-#include "empathy-chat-manager.h"
 #include "empathy-status-icon.h"
 #include "empathy-ft-manager.h"
 
@@ -73,13 +71,309 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
-static gboolean start_hidden = FALSE;
-static gboolean no_connect = FALSE;
+#define EMPATHY_DBUS_NAME "org.gnome.Empathy"
+
+#define EMPATHY_TYPE_APP (empathy_app_get_type ())
+#define EMPATHY_APP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EMPATHY_TYPE_APP, EmpathyApp))
+#define EMPATHY_APP_CLASS(obj) (G_TYPE_CHECK_CLASS_CAST ((obj), EMPATHY_TYPE_APP, EmpathyAppClass))
+#define EMPATHY_IS_EMPATHY_APP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EMPATHY_TYPE_APP))
+#define EMPATHY_IS_EMPATHY_APP_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), EMPATHY_TYPE_APP))
+#define EMPATHY_APP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EMPATHY_TYPE_APP, EmpathyAppClass))
+
+typedef struct _EmpathyApp EmpathyApp;
+typedef struct _EmpathyAppClass EmpathyAppClass;
+
+enum
+{
+  PROP_NO_CONNECT = 1,
+  PROP_START_HIDDEN
+};
+
+GType empathy_app_get_type (void);
+
+struct _EmpathyAppClass
+{
+  GtkApplicationClass parent_class;
+};
+
+struct _EmpathyApp
+{
+  GtkApplication parent;
+
+  /* Properties */
+  gboolean no_connect;
+  gboolean start_hidden;
+
+  gboolean activated;
+
+  GtkWidget *window;
+  EmpathyStatusIcon *icon;
+  EmpathyDispatcher *dispatcher;
+  TpAccountManager *account_manager;
+  TplLogManager *log_manager;
+  EmpathyChatroomManager *chatroom_manager;
+  EmpathyFTFactory  *ft_factory;
+  EmpathyIdle *idle;
+  EmpathyConnectivity *connectivity;
+  GSettings *gsettings;
+#ifdef HAVE_GEOCLUE
+  EmpathyLocationManager *location_manager;
+#endif
+#ifdef ENABLE_DEBUG
+  TpDebugSender *debug_sender;
+#endif
+};
+
+
+G_DEFINE_TYPE(EmpathyApp, empathy_app, GTK_TYPE_APPLICATION)
+
+static void
+empathy_app_dispose (GObject *object)
+{
+  EmpathyApp *self = EMPATHY_APP (object);
+  void (*dispose) (GObject *) =
+    G_OBJECT_CLASS (empathy_app_parent_class)->dispose;
+
+  if (self->idle != NULL)
+    {
+      empathy_idle_set_state (self->idle, TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
+    }
+
+#ifdef ENABLE_DEBUG
+  tp_clear_object (&self->debug_sender);
+#endif
+
+  tp_clear_object (&self->idle);
+  tp_clear_object (&self->connectivity);
+  tp_clear_object (&self->icon);
+  tp_clear_object (&self->account_manager);
+  tp_clear_object (&self->log_manager);
+  tp_clear_object (&self->dispatcher);
+  tp_clear_object (&self->chatroom_manager);
+#ifdef HAVE_GEOCLUE
+  tp_clear_object (&self->location_manager);
+#endif
+  tp_clear_object (&self->ft_factory);
+  tp_clear_object (&self->gsettings);
+
+  if (dispose != NULL)
+    dispose (object);
+}
+
+static void
+empathy_app_finalize (GObject *object)
+{
+  EmpathyApp *self = EMPATHY_APP (object);
+  void (*finalize) (GObject *) =
+    G_OBJECT_CLASS (empathy_app_parent_class)->finalize;
+
+  if (self->window != NULL)
+    gtk_widget_destroy (self->window);
+
+  if (finalize != NULL)
+    finalize (object);
+}
 
 static void account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data);
 
+static void
+empathy_app_set_property (GObject *object,
+    guint prop_id,
+    const GValue *value,
+    GParamSpec *pspec)
+{
+  EmpathyApp *self = EMPATHY_APP (object);
+
+  switch (prop_id)
+    {
+      case PROP_NO_CONNECT:
+        self->no_connect = g_value_get_boolean (value);
+        break;
+      case PROP_START_HIDDEN:
+        self->start_hidden = g_value_get_boolean (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+static void
+new_incoming_transfer_cb (EmpathyFTFactory *factory,
+    EmpathyFTHandler *handler,
+    GError *error,
+    gpointer user_data)
+{
+  if (error)
+    empathy_ft_manager_display_error (handler, error);
+  else
+    empathy_receive_file_with_file_chooser (handler);
+}
+
+static void
+new_ft_handler_cb (EmpathyFTFactory *factory,
+    EmpathyFTHandler *handler,
+    GError *error,
+    gpointer user_data)
+{
+  if (error)
+    empathy_ft_manager_display_error (handler, error);
+  else
+    empathy_ft_manager_add_handler (handler);
+
+  g_object_unref (handler);
+}
+
+static int
+empathy_app_command_line (GApplication *app,
+    GApplicationCommandLine *cmdline)
+{
+  EmpathyApp *self = (EmpathyApp *) app;
+
+  if (!self->activated)
+    {
+      GError *error = NULL;
+
+      /* Create the FT factory */
+      self->ft_factory = empathy_ft_factory_dup_singleton ();
+      g_signal_connect (self->ft_factory, "new-ft-handler",
+          G_CALLBACK (new_ft_handler_cb), NULL);
+      g_signal_connect (self->ft_factory, "new-incoming-transfer",
+          G_CALLBACK (new_incoming_transfer_cb), NULL);
+
+      if (!empathy_ft_factory_register (self->ft_factory, &error))
+        {
+          g_warning ("Failed to register FileTransfer handler: %s",
+              error->message);
+          g_error_free (error);
+        }
+
+      g_application_hold (G_APPLICATION (app));
+      self->activated = TRUE;
+
+      /* Setting up UI */
+      self->window = empathy_main_window_dup ();
+      gtk_widget_show (self->window);
+      self->icon = empathy_status_icon_new (GTK_WINDOW (self->window),
+          self->start_hidden);
+    }
+  else
+    {
+      /* We're requested to show stuff again, disable the start hidden global in
+       * case the accounts wizard wants to pop up.
+       */
+      self->start_hidden = FALSE;
+    }
+
+  if (!self->start_hidden)
+    empathy_window_present (GTK_WINDOW (self->window));
+
+  /* Display the accounts dialog if needed */
+  tp_account_manager_prepare_async (self->account_manager, NULL,
+      account_manager_ready_cb, self);
+
+  return 0;
+}
+
+static gboolean
+show_version_cb (const char *option_name,
+    const char *value,
+    gpointer data,
+    GError **error);
+
+static gboolean
+empathy_app_local_command_line (GApplication *app,
+    gchar ***arguments,
+    gint *exit_status)
+{
+  EmpathyApp *self = (EmpathyApp *) app;
+  gint i;
+  gchar **argv;
+  gint argc = 0;
+  gboolean retval = FALSE;
+  GError *error = NULL;
+  gboolean no_connect = FALSE, start_hidden = FALSE;
+
+  GOptionContext *optcontext;
+  GOptionEntry options[] = {
+      { "no-connect", 'n',
+        0, G_OPTION_ARG_NONE, &no_connect,
+        N_("Don't connect on startup"),
+        NULL },
+      { "start-hidden", 'h',
+        0, G_OPTION_ARG_NONE, &start_hidden,
+        N_("Don't display the contact list or any other dialogs on startup"),
+        NULL },
+      { "version", 'v',
+        G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb,
+        NULL, NULL },
+      { NULL }
+  };
+
+  optcontext = g_option_context_new (N_("- Empathy IM Client"));
+  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
+  g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
+
+  argv = *arguments;
+  for (i = 0; argv[i] != NULL; i++)
+    argc++;
+
+  if (!g_option_context_parse (optcontext, &argc, &argv, &error))
+    {
+      g_print ("%s\nRun '%s --help' to see a full list of available command "
+          "line options.\n",
+          error->message, argv[0]);
+      g_warning ("Error in empathy init: %s", error->message);
+
+      *exit_status = EXIT_FAILURE;
+      retval = TRUE;
+    }
+
+  g_option_context_free (optcontext);
+
+  self->no_connect = no_connect;
+  self->start_hidden = start_hidden;
+
+  return retval;
+}
+
+static void empathy_app_constructed (GObject *object);
+
+static void
+empathy_app_class_init (EmpathyAppClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GApplicationClass *g_app_class = G_APPLICATION_CLASS (klass);
+  GParamSpec *spec;
+
+  gobject_class->set_property = empathy_app_set_property;
+  gobject_class->constructed = empathy_app_constructed;
+  gobject_class->dispose = empathy_app_dispose;
+  gobject_class->finalize = empathy_app_finalize;
+
+  g_app_class->command_line = empathy_app_command_line;
+  g_app_class->local_command_line = empathy_app_local_command_line;
+
+  spec = g_param_spec_boolean ("no-connect", "no connect",
+      "Don't connect on startup",
+      FALSE,
+      G_PARAM_STATIC_STRINGS | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  g_object_class_install_property (gobject_class, PROP_NO_CONNECT, spec);
+
+  spec = g_param_spec_boolean ("start-hidden", "start hidden",
+      "Don't display the contact list or any other dialogs on startup",
+      FALSE,
+      G_PARAM_STATIC_STRINGS | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  g_object_class_install_property (gobject_class, PROP_START_HIDDEN, spec);
+}
+
+static void
+empathy_app_init (EmpathyApp *self)
+{
+}
+
 static void
 use_conn_notify_cb (GSettings *gsettings,
     const gchar *key,
@@ -160,54 +454,14 @@ migrate_config_to_xdg_dir (void)
 }
 
 static void
-show_accounts_ui (GdkScreen *screen,
+show_accounts_ui (EmpathyApp *self,
+    GdkScreen *screen,
     gboolean if_needed)
 {
   empathy_accounts_dialog_show_application (screen,
-      NULL, if_needed, start_hidden);
+      NULL, if_needed, self->start_hidden);
 }
 
-static UniqueResponse
-unique_app_message_cb (UniqueApp *unique_app,
-    gint command,
-    UniqueMessageData *data,
-    guint timestamp,
-    gpointer user_data)
-{
-  GtkWindow *window = user_data;
-  TpAccountManager *account_manager;
-
-  DEBUG ("Other instance launched, presenting the main window. "
-      "Command=%d, timestamp %u", command, timestamp);
-
-  /* XXX: the standalone app somewhat breaks this case, since
-   * communicating it would be a pain */
-
-  /* We're requested to show stuff again, disable the start hidden global
-   * in case the accounts wizard wants to pop up.
-   */
-  start_hidden = FALSE;
-
-  gtk_window_set_screen (GTK_WINDOW (window),
-      unique_message_data_get_screen (data));
-  gtk_window_set_startup_id (GTK_WINDOW (window),
-      unique_message_data_get_startup_id (data));
-  gtk_window_present_with_time (GTK_WINDOW (window), timestamp);
-  gtk_window_set_skip_taskbar_hint (window, FALSE);
-
-  account_manager = tp_account_manager_dup ();
-  tp_account_manager_prepare_async (account_manager, NULL,
-      account_manager_ready_cb, NULL);
-  g_object_unref (account_manager);
-
-  return UNIQUE_RESPONSE_OK;
-}
-
-static gboolean show_version_cb (const char *option_name,
-    const char *value,
-    gpointer data,
-    GError **error) G_GNUC_NORETURN;
-
 static gboolean
 show_version_cb (const char *option_name,
     const char *value,
@@ -219,73 +473,53 @@ show_version_cb (const char *option_name,
   exit (EXIT_SUCCESS);
 }
 
-static void
-new_incoming_transfer_cb (EmpathyFTFactory *factory,
-    EmpathyFTHandler *handler,
-    GError *error,
-    gpointer user_data)
-{
-  if (error)
-    empathy_ft_manager_display_error (handler, error);
-  else
-    empathy_receive_file_with_file_chooser (handler);
-}
-
-static void
-new_ft_handler_cb (EmpathyFTFactory *factory,
-    EmpathyFTHandler *handler,
-    GError *error,
-    gpointer user_data)
-{
-  if (error)
-    empathy_ft_manager_display_error (handler, error);
-  else
-    empathy_ft_manager_add_handler (handler);
-
-  g_object_unref (handler);
-}
-
 static void
 account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data)
 {
   TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+  EmpathyApp *self = user_data;
   GError *error = NULL;
-  EmpathyIdle *idle;
-  EmpathyConnectivity *connectivity;
   TpConnectionPresenceType presence;
-  GSettings *gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
 
   if (!tp_account_manager_prepare_finish (manager, result, &error))
     {
+      GtkWidget *dialog;
+
       DEBUG ("Failed to prepare account manager: %s", error->message);
+
+      dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+          GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+          _("Error contacting the Account Manager"));
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+          _("There was an error while trying to connect to the Telepathy "
+            "Account Manager. The error was:\n\n%s"),
+          error->message);
+
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
       g_error_free (error);
       return;
     }
 
   /* Autoconnect */
-  idle = empathy_idle_dup_singleton ();
-  connectivity = empathy_connectivity_dup_singleton ();
-
   presence = tp_account_manager_get_most_available_presence (manager, NULL,
       NULL);
 
-  if (g_settings_get_boolean (gsettings, EMPATHY_PREFS_AUTOCONNECT) &&
-      !no_connect &&
+  if (g_settings_get_boolean (self->gsettings, EMPATHY_PREFS_AUTOCONNECT) &&
+      !self->no_connect &&
       tp_connection_presence_type_cmp_availability
           (presence, TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
             <= 0)
       /* if current state is Offline, then put it online */
-      empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
+      empathy_idle_set_state (self->idle,
+          TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
 
   /* Pop up the accounts dialog if we don't have any account */
   if (!empathy_accounts_has_accounts (manager))
-    show_accounts_ui (gdk_screen_get_default (), TRUE);
-
-  g_object_unref (idle);
-  g_object_unref (connectivity);
-  g_object_unref (gsettings);
+    show_accounts_ui (self, gdk_screen_get_default (), TRUE);
 }
 
 static void
@@ -382,70 +616,13 @@ empathy_idle_set_auto_away_cb (GSettings *gsettings,
       g_settings_get_boolean (gsettings, key));
 }
 
-int
-main (int argc, char *argv[])
+static void
+empathy_app_constructed (GObject *object)
 {
-#ifdef HAVE_GEOCLUE
-  EmpathyLocationManager *location_manager = NULL;
-#endif
-  EmpathyStatusIcon *icon;
-  EmpathyDispatcher *dispatcher;
-  TpAccountManager *account_manager;
-  TplLogManager *log_manager;
-  EmpathyChatroomManager *chatroom_manager;
-  EmpathyFTFactory  *ft_factory;
-  GtkWidget *window;
-  EmpathyIdle *idle;
-  EmpathyConnectivity *connectivity;
-  EmpathyChatManager *chat_manager;
-  GError *error = NULL;
-  UniqueApp *unique_app;
+  EmpathyApp *self = (EmpathyApp *) object;
   gboolean chatroom_manager_ready;
-  gboolean autoaway = TRUE;
-#ifdef ENABLE_DEBUG
-  TpDebugSender *debug_sender;
-#endif
-  GSettings *gsettings;
+  gboolean autoaway;
 
-  GOptionContext *optcontext;
-  GOptionEntry options[] = {
-      { "no-connect", 'n',
-        0, G_OPTION_ARG_NONE, &no_connect,
-        N_("Don't connect on startup"),
-        NULL },
-      { "start-hidden", 'h',
-        0, G_OPTION_ARG_NONE, &start_hidden,
-        N_("Don't display the contact list or any other dialogs on startup"),
-        NULL },
-      { "version", 'v',
-        G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb,
-        NULL, NULL },
-      { NULL }
-  };
-
-  /* Init */
-  g_thread_init (NULL);
-
-#ifdef HAVE_LIBCHAMPLAIN
-  gtk_clutter_init (&argc, &argv);
-#endif
-
-  empathy_init ();
-
-  optcontext = g_option_context_new (N_("- Empathy IM Client"));
-  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
-  g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
-
-  if (!g_option_context_parse (optcontext, &argc, &argv, &error)) {
-    g_print ("%s\nRun '%s --help' to see a full list of available command line options.\n",
-        error->message, argv[0]);
-    g_warning ("Error in empathy init: %s", error->message);
-    return EXIT_FAILURE;
-  }
-
-  g_option_context_free (optcontext);
-
-  empathy_gtk_init ();
   g_set_application_name (_(PACKAGE_NAME));
 
   gtk_window_set_default_icon_name ("empathy");
@@ -453,124 +630,98 @@ main (int argc, char *argv[])
 
 #ifdef ENABLE_DEBUG
   /* Set up debug sender */
-  debug_sender = tp_debug_sender_dup ();
+  self->debug_sender = tp_debug_sender_dup ();
   g_log_set_default_handler (tp_debug_sender_log_handler, G_LOG_DOMAIN);
 #endif
 
-  unique_app = unique_app_new ("org.gnome."PACKAGE_NAME, NULL);
-
-  if (unique_app_is_running (unique_app))
-    {
-      unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
-
-      g_object_unref (unique_app);
-      return EXIT_SUCCESS;
-    }
-
   notify_init (_(PACKAGE_NAME));
 
   /* Setting up Idle */
-  idle = empathy_idle_dup_singleton ();
+  self->idle = empathy_idle_dup_singleton ();
 
-  gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
-  autoaway = g_settings_get_boolean (gsettings, EMPATHY_PREFS_AUTOAWAY);
+  self->gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
+  autoaway = g_settings_get_boolean (self->gsettings, EMPATHY_PREFS_AUTOAWAY);
 
-  g_signal_connect (gsettings,
+  g_signal_connect (self->gsettings,
       "changed::" EMPATHY_PREFS_AUTOAWAY,
-      G_CALLBACK (empathy_idle_set_auto_away_cb), idle);
+      G_CALLBACK (empathy_idle_set_auto_away_cb), self->idle);
 
-  empathy_idle_set_auto_away (idle, autoaway);
+  empathy_idle_set_auto_away (self->idle, autoaway);
 
   /* Setting up Connectivity */
-  connectivity = empathy_connectivity_dup_singleton ();
-  use_conn_notify_cb (gsettings, EMPATHY_PREFS_USE_CONN,
-      connectivity);
-  g_signal_connect (gsettings,
+  self->connectivity = empathy_connectivity_dup_singleton ();
+  use_conn_notify_cb (self->gsettings, EMPATHY_PREFS_USE_CONN,
+      self->connectivity);
+  g_signal_connect (self->gsettings,
       "changed::" EMPATHY_PREFS_USE_CONN,
-      G_CALLBACK (use_conn_notify_cb), connectivity);
+      G_CALLBACK (use_conn_notify_cb), self->connectivity);
 
   /* account management */
-  account_manager = tp_account_manager_dup ();
-  tp_account_manager_prepare_async (account_manager, NULL,
-      account_manager_ready_cb, NULL);
+  self->account_manager = tp_account_manager_dup ();
+  tp_account_manager_prepare_async (self->account_manager, NULL,
+      account_manager_ready_cb, self);
 
   /* The EmpathyDispatcher doesn't dispatch anything any more but we have to
    * keep it around as we still use it to request channels */
-  dispatcher = empathy_dispatcher_dup_singleton ();
+  self->dispatcher = empathy_dispatcher_dup_singleton ();
 
   migrate_config_to_xdg_dir ();
 
-  /* Setting up UI */
-  window = empathy_main_window_dup ();
-  gtk_widget_show (window);
-  icon = empathy_status_icon_new (GTK_WINDOW (window), start_hidden);
-
-  /* Chat manager */
-  chat_manager = empathy_chat_manager_dup_singleton ();
-
-  g_signal_connect (unique_app, "message-received",
-      G_CALLBACK (unique_app_message_cb), window);
-
   /* Logging */
-  log_manager = tpl_log_manager_dup_singleton ();
+  self->log_manager = tpl_log_manager_dup_singleton ();
 
-  chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
+  self->chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 
-  g_object_get (chatroom_manager, "ready", &chatroom_manager_ready, NULL);
+  g_object_get (self->chatroom_manager, "ready", &chatroom_manager_ready, NULL);
   if (!chatroom_manager_ready)
     {
-      g_signal_connect (G_OBJECT (chatroom_manager), "notify::ready",
-          G_CALLBACK (chatroom_manager_ready_cb), account_manager);
+      g_signal_connect (G_OBJECT (self->chatroom_manager), "notify::ready",
+          G_CALLBACK (chatroom_manager_ready_cb), self->account_manager);
     }
   else
     {
-      chatroom_manager_ready_cb (chatroom_manager, NULL, account_manager);
-    }
-
-  /* Create the FT factory */
-  ft_factory = empathy_ft_factory_dup_singleton ();
-  g_signal_connect (ft_factory, "new-ft-handler",
-      G_CALLBACK (new_ft_handler_cb), NULL);
-  g_signal_connect (ft_factory, "new-incoming-transfer",
-      G_CALLBACK (new_incoming_transfer_cb), NULL);
-
-  if (!empathy_ft_factory_register (ft_factory, &error))
-    {
-      g_warning ("Failed to register FileTransfer handler: %s", error->message);
-      g_error_free (error);
+      chatroom_manager_ready_cb (self->chatroom_manager, NULL,
+          self->account_manager);
     }
 
   /* Location mananger */
 #ifdef HAVE_GEOCLUE
-  location_manager = empathy_location_manager_dup_singleton ();
+  self->location_manager = empathy_location_manager_dup_singleton ();
 #endif
 
-  gtk_main ();
+  self->activated = FALSE;
+  self->ft_factory = NULL;
+  self->window = NULL;
+}
 
-  empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
+int
+main (int argc, char *argv[])
+{
+  EmpathyApp *app;
+  gint retval;
 
-#ifdef ENABLE_DEBUG
-  g_object_unref (debug_sender);
+  g_thread_init (NULL);
+  g_type_init ();
+
+#ifdef HAVE_LIBCHAMPLAIN
+  gtk_clutter_init (&argc, &argv);
 #endif
 
-  g_object_unref (chat_manager);
-  g_object_unref (idle);
-  g_object_unref (connectivity);
-  g_object_unref (icon);
-  g_object_unref (account_manager);
-  g_object_unref (log_manager);
-  g_object_unref (dispatcher);
-  g_object_unref (chatroom_manager);
-#ifdef HAVE_GEOCLUE
-  g_object_unref (location_manager);
-#endif
-  g_object_unref (ft_factory);
-  g_object_unref (unique_app);
-  g_object_unref (gsettings);
-  gtk_widget_destroy (window);
+  empathy_init ();
+  gtk_init (&argc, &argv);
+  empathy_gtk_init ();
+
+  app = g_object_new (EMPATHY_TYPE_APP,
+      "application-id", EMPATHY_DBUS_NAME,
+      "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
+      NULL);
+
+  retval = g_application_run (G_APPLICATION (app), argc, argv);
 
   notify_uninit ();
   xmlCleanupParser ();
 
-  return EXIT_SUCCESS;
+  g_object_unref (app);
+
+  return retval;
 }
